package plugins

import BuildAndroidConfig
import BuildModules
import BuildPlugins
import com.android.build.gradle.AppExtension
import dependencies.DebugDependencies
import dependencies.Dependencies
import dependencies.TekoDependencies
import org.gradle.api.JavaVersion
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.plugins.ExtensionAware
import org.gradle.kotlin.dsl.apply
import org.gradle.kotlin.dsl.dependencies
import org.gradle.kotlin.dsl.project
import org.gradle.plugin.use.PluginDependenciesSpec
import org.gradle.plugin.use.PluginDependencySpec
import org.jetbrains.kotlin.gradle.dsl.KotlinJvmOptions
import utils.getLocalProperty

/**
 * Gradle plugin implemented by [SmartPosApplicationPlugin].
 *
 * @see [SmartPosApplicationPlugin]
 */
inline val PluginDependenciesSpec.`smart-pos-application`: PluginDependencySpec
    get() = id("smart-pos-application")

class SmartPosApplicationPlugin : Plugin<Project> {

    override fun apply(target: Project): Unit = target.run {
        configApplication()
        configDependencies()
    }

    private fun Project.configApplication() = run {
        apply(plugin = BuildPlugins.ANDROID_APPLICATION)
        apply(plugin = BuildPlugins.KOTLIN_ANDROID)
        apply(plugin = BuildPlugins.KOTLIN_KAPT)

        android {
            compileSdkVersion(BuildAndroidConfig.COMPILE_SDK_VERSION)
            buildToolsVersion(BuildAndroidConfig.BUILD_TOOLS_VERSION)

            defaultConfig {
                minSdkVersion(BuildAndroidConfig.MIN_SDK_VERSION)
                targetSdkVersion(BuildAndroidConfig.TARGET_SDK_VERSION)

                versionName = BuildAndroidConfig.VERSION_NAME

                testInstrumentationRunner = BuildAndroidConfig.TEST_INSTRUMENTATION_RUNNER

                vectorDrawables.useSupportLibrary = true

                flavorDimensions(flavors.BuildProductDimensions.ENVIRONMENT)

                multiDexEnabled = BuildAndroidConfig.MULTIDEX_ENABLED

                testOptions {
                    android {
                        multiDexEnabled = true
                    }
                }
            }

            signingConfigs {
                getByName("debug") {
                    keyAlias = "debug"
                    keyPassword = "Abc@123"
                    storeFile = file(this@run.rootDir.absolutePath + "/buildsystem/debug")
                    storePassword = "Abc@123"
                }
                create("release") {
                    keyAlias = getLocalProperty(this@run, "signing.key.alias")
                    keyPassword = getLocalProperty(this@run, "signing.key.password")
                    storeFile = file(this@run.rootDir.absolutePath + "/buildsystem/release")
                    storePassword = getLocalProperty(this@run, "signing.store.password")
                }
            }

            buildTypes {
                getByName("debug") {
                    isMinifyEnabled = false
                    isTestCoverageEnabled = System.getenv("TEST_COVERAGE_ENABLED") == "true"
                    signingConfig = signingConfigs.getByName("debug")
                }
                getByName("release") {
                    isDebuggable = false
                    isMinifyEnabled = true
                    isShrinkResources = true
                    proguardFiles(
                        getDefaultProguardFile("proguard-android-optimize.txt"),
                        "proguard-rules.pro"
                    )
                    signingConfig = signingConfigs.getByName("release")
                }
            }

            compileOptions {
                isCoreLibraryDesugaringEnabled = true

                sourceCompatibility = JavaVersion.VERSION_1_8
                targetCompatibility = JavaVersion.VERSION_1_8
            }

            kotlinOptions {
                jvmTarget = JavaVersion.VERSION_1_8.toString()
            }

            packagingOptions {
                exclude("META-INF/*.kotlin_module")
                exclude("META-INF/spring.tooling")
                exclude("META-INF/NOTICE.md")
                exclude("META-INF/spring.handlers")
                exclude("META-INF/spring-configuration-metadata.json")
                exclude("META-INF/additional-spring-configuration-metadata.json")
                exclude("META-INF/spring.factories")
                exclude("META-INF/spring.schemas")
                exclude("META-INF/license.txt")
                exclude("META-INF/notice.txt")
                exclude("META-INF/LICENSE.md")
                exclude("META-INF/AL2.0")
                exclude("META-INF/LGPL2.1")

                pickFirst("lib/armeabi-v7a/libc++_shared.so")
                pickFirst("lib/arm64-v8a/libc++_shared.so")
                pickFirst("lib/x86/libc++_shared.so")
                pickFirst("lib/x86_64/libc++_shared.so")
                pickFirst("lib/x86_64/libfbjni.so")
                pickFirst("lib/x86_64/libfolly_futures.so")
                pickFirst("lib/arm64-v8a/libjsijniprofiler.so")
                pickFirst("lib/arm64-v8a/libhermes-executor-debug.so")
                pickFirst("lib/**/*.so")
            }
        }
    }

    private fun Project.configDependencies() = dependencies {
        add("coreLibraryDesugaring", Dependencies.DESUGAR_JDK_LIBS)
        add("implementation", project(":apps:staff:base"))

        add(
            "implementation", fileTree(
                mapOf(
                    "dir" to "libs",
                    "include" to listOf("*.jar", "*.aar")
                )
            )
        )

        if (BuildAndroidConfig.USE_PREBUILT_LIBRARIES) {
            add("implementation", TekoDependencies.TERRA_CORE_ANDROID)

            // Apollo
            add("implementation", TekoDependencies.TERRA_APOLLO)
        } else {
            add("implementation", project(BuildModules.TERRA_CORE_ANDROID))

            // Apollo
            add("implementation", project(BuildModules.TERRA_APOLLO))
        }

        // for debug only
        add("debugImplementation", DebugDependencies.LEAK_CANARY)
        add("debugImplementation", DebugDependencies.DEBUG_DB)
    }

    private fun Project.android(configure: AppExtension.() -> Unit): Unit =
        (this as ExtensionAware).extensions.configure("android", configure)

    private fun AppExtension.kotlinOptions(configure: KotlinJvmOptions.() -> Unit): Unit =
        (this as ExtensionAware).extensions.configure("kotlinOptions", configure)
}
