package vn.teko.sandbox.nativetest.ui.dashboard

import android.app.ProgressDialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import vn.teko.android.payment.kit.PaymentKit
import vn.teko.android.payment.kit.PaymentResultV2Callback
import vn.teko.android.payment.kit.model.request.PaymentRequestBuilder
import vn.teko.android.payment.kit.model.request.PaymentV2Request
import vn.teko.android.payment.kit.model.result.PaymentV2Result
import vn.teko.sandbox.nativetest.R
import vn.teko.sandbox.nativetest.cart.CartManager
import vn.teko.sandbox.nativetest.databinding.FragmentCartBinding
import vn.teko.sandbox.nativetest.utils.MoneyFormatter
import kotlin.math.roundToLong

class CartFragment : Fragment() {
    private lateinit var cartItemListAdapter: CartItemListAdapter
    private lateinit var linearLayoutManager: LinearLayoutManager

    private lateinit var dataBinding: FragmentCartBinding

    private val orderCode: String = System.currentTimeMillis().toString()

    private var progressDialog: ProgressDialog? = null

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        dataBinding = DataBindingUtil.inflate(
            inflater, R.layout.fragment_cart, container, false
        )
        setupView()
        return dataBinding.root
    }

    private fun getGrandTotal() = CartManager.getItems().sumOf { it.totalPrice }

    private fun setupView() {
        linearLayoutManager = LinearLayoutManager(requireContext())
        cartItemListAdapter = CartItemListAdapter()
        dataBinding.rcvCartItems.adapter = cartItemListAdapter
        dataBinding.rcvCartItems.layoutManager = linearLayoutManager
        updateCartState()

        dataBinding.btnCheckout.setOnClickListener {
            val message =
                getString(R.string.payment_confirm, dataBinding.tvGrandTotal.text, orderCode)
            AlertDialog.Builder(requireContext())
                .setMessage(message)
                .setPositiveButton(R.string.yes) { _, _ ->
                    performPayment()
                }
                .setNegativeButton(R.string.cancel) { _, _ -> }
                .setCancelable(false)
                .show()
        }
    }

    private fun updateCartState() {
        dataBinding.tvGrandTotal.text = MoneyFormatter.formatVND(getGrandTotal())
        cartItemListAdapter.setItems(CartManager.getItems())
    }

    private fun performPayment() {
        val grantTotal = getGrandTotal().toLong()
        if (grantTotal <= 0) return

        val paymentCallbackV2 = object : PaymentResultV2Callback {
            override fun onResult(result: PaymentV2Result) {
                hideLoading()
                val paymentResultStatus = when (result.resultCode) {
                    PaymentV2Result.RESULT_CANCELED -> R.string.cancelled_payment
                    PaymentV2Result.RESULT_FAILED -> R.string.failed_payment
                    else -> {
                        CartManager.clear()
                        updateCartState()
                        R.string.success_payment
                    }
                }

                AlertDialog.Builder(requireContext())
                    .setMessage(paymentResultStatus)
                    .setPositiveButton(R.string.OK) { _, _ -> }
                    .create()
                    .show()
            }
        }

        val paymentRequestBuilder =
            PaymentRequestBuilder(
                PaymentV2Request.Order(
                    orderCode = orderCode,
                    orderAmount = getGrandTotal().roundToLong(),
                    displayOrderCode = DISPLAY_ORDER_CODE
                )
            )
                .setMainMethod(
                    PaymentV2Request.Payment.MainMethod(
                        PaymentV2Request.Payment.Method.All,
                        grantTotal,
                    )
                )

        showLoading()
        PaymentKit.payForOrder(
            null,
            paymentRequest = paymentRequestBuilder.build(),
            callback = paymentCallbackV2
        )
    }

    private fun showLoading() {
        context?.also {
            if (progressDialog == null) {
                progressDialog = ProgressDialog(it)
                progressDialog?.setTitle("Loading")
                progressDialog?.show();
            } else if (progressDialog?.isShowing == false) {
                progressDialog!!.show()
            }
        }
    }

    /**
     * cleanup any loading behavior has taken before
     */
    private fun hideLoading() {
        progressDialog?.dismiss()
        progressDialog = null
    }

    companion object {
        private const val DISPLAY_ORDER_CODE = "Sandbox-native-mini-app"
    }
}
